import 'dart:io';
import 'package:dio/dio.dart';
import 'package:firebase_storage/firebase_storage.dart';
import 'package:bmi_global/provider/firebase_doc.dart';
import 'package:bmi_global/utils/parsers.dart';
import 'package:bmi_global/utils/version.dart';
import 'package:open_file/open_file.dart';

AppInfoDoc Function(RawMap) _parse(String node) {
  return (data) {
    return AppInfoDoc.fromJson(data[node].toMap());
  };
}

class AppInfoProvider extends FirebaseDocProvider<AppInfoDoc> {
  var downloading = false;
  var progress = .0;
  var apkAvalable = false;
  final Version appVersion;
  final Directory _apkDir;
  final String _listenNode;

  AppInfoProvider({
    required String listenNode,
    required this.appVersion,
    required Directory apkDir,
  })  : _apkDir = apkDir,
        _listenNode = listenNode,
        super(parse: _parse(listenNode)) {
    _init();
  }

  @override
  String? get path => "CONFIG/ANDROID_APK_LOG";

  final storageInstance = FirebaseStorage.instanceFor(
    app: FirebaseStorage.instance.app,
    bucket: "bmi-apks",
  );

  Version get currentVersion => doc?.version ?? appVersion;

  String get _filePath => "${_apkDir.path}/$currentVersion.apk";

  @override
  AppInfoDoc? convertor(Map<String, Object?>? data) {
    final newDoc = super.convertor(data);
    if (newDoc?.version != doc?.version) _checkApk();
    return newDoc;
  }

  bool get isNew {
    return currentVersion > appVersion;
  }

  void _init() async {
    final list = (await _apkDir.list().toList()).whereType<File>();
    for (var file in list) {
      final x = file.uri.pathSegments.last;
      final y = x.substring(0, x.length - 4).split("+");
      final fileVersion = Version(y.first.split("."), y.last);
      if (fileVersion <= appVersion) file.delete();
    }
  }

  void download() async {
    try {
      final dio = Dio();
      await dio.download(
        await storageInstance.ref("$_listenNode.apk").getDownloadURL(),
        _filePath,
        onReceiveProgress: (rec, total) {
          downloading = true;
          progress = rec / total;
          notifyListeners();
        },
      );
      openApk();
    } catch (_) {}
    downloading = false;
    _checkApk();
    notifyListeners();
  }

  void _checkApk() {
    File(_filePath).exists().then((exists) {
      if (exists) apkAvalable = true;
      notifyListeners();
    });
  }

  void openApk() {
    OpenFile.open(_filePath);
  }
}
