import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/material.dart';
import 'package:flutter_auth_ui/flutter_auth_ui.dart';
import 'package:bmi_global/utils/user.dart';

class AuthProvider<U extends AuthUser> with ChangeNotifier {
  var _loading = true;
  final FirebaseAuth _authInstance;
  U? _user;
  late final void Function() _cancle;
  final Future<U> Function(User user) userBuilder;

  final TosAndPrivacyPolicy _tosAndPrivacyPolicy;

  final List<AuthUiProvider> _authProviders;

  bool get loading => _loading;
  U? get user => _user;

  AuthProvider({
    required this.userBuilder,
    TosAndPrivacyPolicy? policy,
    List<AuthUiProvider>? providers,
    FirebaseAuth? authInstance,
  })  : _authInstance = authInstance ?? FirebaseAuth.instance,
        _authProviders = providers ?? [AuthUiProvider.phone],
        _tosAndPrivacyPolicy = policy ??
            const TosAndPrivacyPolicy(
              tosUrl: "https://www.google.com",
              privacyPolicyUrl: "https://www.google.com",
            ) {
    _cancle = _authInstance.userChanges().listen((user) async {
      if (user != null) {
        try {
          _user = await userBuilder(user);
        } catch (_) {
          _user = null;
        }
      }
      _loading = false;
      notifyListeners();
    }).cancel;
  }

  void refresh() {
    _authInstance.currentUser?.reload();
  }

  void logOut() {
    if (_user == null || _loading) return;
    _user = null;
    _authInstance.signOut();
    notifyListeners();
  }

  Future<void> login() async {
    if (_loading) return;
    _loading = true;
    if (!await FlutterAuthUi.startUi(
      items: _authProviders,
      tosAndPrivacyPolicy: _tosAndPrivacyPolicy,
    )) {
      _loading = false;
      notifyListeners();
    }
  }

  @override
  void dispose() {
    _cancle();
    super.dispose();
  }
}
