import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/material.dart';
import 'package:bmi_global/utils/parsers.dart';
import 'package:provider/provider.dart';

abstract class FirebaseDocProvider<T> with ChangeNotifier {
  T? _doc;
  var _loading = false;
  Object? _error;
  final bool Function(Object error)? cancleOnError;
  final T Function(RawMap data) parse;
  void Function()? _cancle;
  String? _path;

  final FirebaseFirestore _firestoreInstance;

  bool get loading => _loading;
  T? get doc => _doc;
  Object? get error => _error;

  FirebaseDocProvider({
    required this.parse,
    this.cancleOnError,
    FirebaseFirestore? firestoreInstance,
  }) : _firestoreInstance = firestoreInstance ?? FirebaseFirestore.instance {
    notifyListeners();
  }

  String? get path;

  @override
  void notifyListeners() {
    final oldPath = _path;
    final newPath = path;
    if (oldPath == newPath) return super.notifyListeners();
    _path = newPath;
    _cancle?.call();
    _doc = null;
    _loading = true;
    _error = null;
    if (newPath != null) {
      _cancle = _firestoreInstance
          .doc(newPath)
          .snapshots()
          .handleError(_onError)
          .listen(_listen, cancelOnError: false)
          .cancel;
    } else {
      _cancle = null;
    }
    super.notifyListeners();
  }

  void _onError(Object error) {
    _loading = false;
    _error = error;
    if (cancleOnError?.call(error) ?? false) {
      _cancle?.call();
      _cancle = null;
    }
    super.notifyListeners();
  }

  T? convertor(Map<String, Object?>? data) {
    if (data == null) return null;
    return parse(RawMap(data));
  }

  void _listen(DocumentSnapshot<Map<String, Object?>> event) {
    _loading = false;
    _doc = convertor(event.data());
    super.notifyListeners();
  }

  @override
  void dispose() {
    _cancle?.call();
    super.dispose();
  }
}

ProxyProvider<DocProvider<T>, T> providerDoc<T>({
  Widget Function(BuildContext, Widget?)? builder,
  Widget? child,
  T Function(BuildContext)? create,
  void Function(BuildContext, T)? dispose,
  Key? key,
  bool? lazy,
  bool Function(T, T)? updateShouldNotify,
}) {
  return ProxyProvider(
    update: (_, doc, __) => doc.doc!,
    builder: builder,
    create: create,
    dispose: dispose,
    key: key,
    lazy: lazy,
    updateShouldNotify: updateShouldNotify,
    child: child,
  );
}

class DocProvider<T> with ChangeNotifier {
  T? _doc;
  var _loading = false;
  Object? _error;
  final bool Function(Object error)? cancleOnError;
  final T Function(RawMap data) parse;
  void Function()? _cancle;
  String? _path;

  final FirebaseFirestore _firestoreInstance;

  bool get loading => _loading;
  T? get doc => _doc;
  Object? get error => _error;

  DocProvider({
    required this.parse,
    this.cancleOnError,
    FirebaseFirestore? firestoreInstance,
    String? initialPath,
  }) : _firestoreInstance = firestoreInstance ?? FirebaseFirestore.instance {
    if (initialPath != null) changePath(initialPath);
  }

  void changePath(String? newPath) {
    final oldPath = _path;
    if (oldPath == newPath) return;
    _path = newPath;
    _cancle?.call();
    _doc = null;
    _loading = true;
    _error = null;
    if (newPath != null) {
      _cancle = _firestoreInstance
          .doc(newPath)
          .snapshots()
          .handleError(_onError)
          .listen(_listen, cancelOnError: false)
          .cancel;
    } else {
      _cancle = null;
    }
    notifyListeners();
  }

  void _onError(Object error) {
    _loading = false;
    _error = error;
    if (cancleOnError?.call(error) ?? false) {
      _cancle?.call();
      _cancle = null;
    }
    notifyListeners();
  }

  T? convertor(Map<String, Object?>? data) {
    if (data == null) return null;
    return parse(RawMap(data));
  }

  void _listen(DocumentSnapshot<Map<String, Object?>> event) {
    _loading = false;
    _doc = convertor(event.data());
    notifyListeners();
  }

  @override
  void dispose() {
    _cancle?.call();
    super.dispose();
  }
}
