import 'package:flutter/material.dart';
import 'package:bmi_global/utils/parsers.dart';
import 'package:intl/intl.dart';

abstract class Dates {
  static List<DateTime> fromRange(DateTimeRange dateTimeRange) {
    final start = dateTimeRange.start;
    final end = dateTimeRange.end;
    final days = <DateTime>[];
    for (int i = 0; i <= end.difference(start).inDays; i++) {
      days.add(start.add(Duration(days: i)));
    }
    return days;
  }

  static List<DateTime> between(DateTime start, DateTime end) {
    switch (start.compareTo(end)) {
      case 1:
        throw Error.throwWithStackTrace(
          "start can't be greator then end",
          StackTrace.current,
        );
      case 0:
        return [];
    }
    final days = <DateTime>[];
    for (int i = 0; i <= end.difference(start).inDays; i++) {
      days.add(start.add(Duration(days: i)));
    }
    return days;
  }
}

class DateTimeString extends Comparable<DateTimeString> {
  final String value;
  final DateTime dateTime;

  static final _nameDateFormate = DateFormat('dd MMM yyyy');
  static final _amPmTimeFormate = DateFormat('hh:mm:ss a');

  DateTimeString.fromDateTime(this.dateTime)
      : value = dateTime.toIso8601String();

  DateTimeString(this.value)
      : dateTime = value.length >= 19
            ? DateTime(
                toInt(value.substring(0, 4)),
                toInt(value.substring(5, 7)),
                toInt(value.substring(8, 10)),
                toInt(value.substring(11, 13)),
                toInt(value.substring(14, 16)),
                toInt(value.substring(17, 19)),
              )
            : value.length >= 10
                ? DateTime(
                    toInt(value.substring(0, 4)),
                    toInt(value.substring(5, 7)),
                    toInt(value.substring(8, 10)),
                  )
                : DateTime(
                    toInt(value.substring(0, 4)),
                    toInt(value.substring(5, 7)),
                  );

  String formateTime({bool withSec = false}) {
    final v = _amPmTimeFormate.format(dateTime);
    if (withSec) return v;
    return v.substring(0, 5) + v.substring(8);
  }

  String formateDate({
    bool name = true,
    bool withYear = true,
    bool withDate = true,
  }) {
    try {
      if (name) {
        return _nameDateFormate
            .format(dateTime)
            .substring(withDate ? 0 : 3, withYear ? null : 6);
      }
      return value.substring(withYear ? 0 : 5, withDate ? 10 : 7);
    } catch (_) {
      return "".padLeft((withYear ? 5 : 0) + 2 + (withDate ? 3 : 0));
    }
  }

  DateTime get firstDayOfMonth => DateTime(dateTime.year, dateTime.month);
  DateTime get lastDayOfMonth =>
      DateTime(dateTime.year, dateTime.month + 1, -1, 24);

  @override
  int compareTo(DateTimeString other) {
    return dateTime.compareTo(other.dateTime);
  }
}

class IntMoney {
  final int money;
  const IntMoney(this.money);

  static const rs = "₹ ";
  static final _amount = NumberFormat("##,##,##,###.##", "en_IN");

  String get textValue => (money / 1000).toString();

  @override
  String toString({bool trail = true, bool lead = true, bool neg = false}) {
    var val = "";
    if (lead) val += rs;
    val += neg ? _amount.format(-money / 1000) : _amount.format(money / 1000);
    if (trail) val += "/-";
    return val;
  }
}

class IntQuntity {
  final int quntity;

  const IntQuntity(this.quntity);

  static final _quntity = NumberFormat("##,##,##,###.###", "en_IN");

  String get textValue => (quntity / 1000).toString();

  @override
  String toString({bool neg = false}) {
    return neg
        ? _quntity.format(-quntity / 1000)
        : _quntity.format(quntity / 1000);
  }
}
